{ Delphi Object for R&R XBASE V6.5 Edition }

{ Written : October 1995 }
{ Property Editors added Jan 1996 }

{ TRRX65 & TRRSQL65 are written and distributed under the GNU 
  General Public License which means that its source code is 
  freely-distributed and available to the general public. }

{ Author Chris Brooksbank (cbrooksbank@dial.pipex.com) }
{ Web Page : http://ds.dial.pipex.com/cbrooksbank }

{ Copyright Chris Brooksbank 1995,1996,1997. }
{ 34 Vermeer Ride
  Springfield
  Chelmsford
  Essex
  CM1 6GA
  United Kingdom } 

unit RRX65;

interface

uses
  SysUtils,WinTypes,WinProcs,Messages,Classes,Graphics,Controls,Forms,
  Dialogs,rrxint, DsgnIntf,FileCtrl, RRSort, StdCtrls, RRUser;

type

  ErrException=Class(Exception);
  ErrHandleAllocation=Class(ErrException);
  ErrCancel=Class(EAbort);
  ErrDiagnostic=Class(ErrException);
  ErrIteration=Class(ErrException);
  ErrJobControl=Class(ErrException);
  ErrLibrary=Class(ErrException);
  ErrSyntax=Class(ErrException);
  ErrValue=Class(ErrException);
  ErrUnKnown=Class(ErrException);
  ErrCodeAbort=Class(EAbort);

  { Custom event procedure types }
  TReportStartEvent=procedure(const ReportHandle:Integer;var AllowToRun:Boolean) of object;
  TReportEndEvent=procedure(const ReportHandle:Integer) of object;

  { Report Library Property Editor Class }
  TRLPropertyEditor=class(TPropertyEditor)
    public
      procedure edit; override;
      function GetAttributes: TPropertyAttributes; override;
      function GetValue:String; override;
      procedure SetValue(Const value:String); override;
  end;

  { Report Name Property Editor Class }
  TRNPropertyEditor=class(tPropertyEditor)
     public
      procedure edit; override;
      function GetAttributes: TPropertyAttributes; override;
      function GetValue:String; override;
      procedure SetValue(Const value:String); override;
  end;

 { Printer Name/Port Property Editor Class}
 TPRPropertyEditor=class(tPropertyEditor)
     public
      procedure edit; override;
      function GetAttributes: TPropertyAttributes; override;
      function GetValue:String; override;
      procedure SetValue(Const value:String); override;
  end;

 { Data Directory Property Editor Class}
 TDDPropertyEditor=class(tPropertyEditor)
     public
      procedure edit; override;
      function GetAttributes: TPropertyAttributes; override;
      function GetValue:String; override;
      procedure SetValue(Const value:String); override;
  end;

 { Image Directory Property Editor Class}
 TIDPropertyEditor=class(tPropertyEditor)
     public
      procedure edit; override;
      function GetAttributes: TPropertyAttributes; override;
      function GetValue:String; override;
      procedure SetValue(Const value:String); override;
  end;

 { MasterTable Property Editor Class}
 TMTPropertyEditor=class(tPropertyEditor)
     public
      procedure edit; override;
      function GetAttributes: TPropertyAttributes; override;
      function GetValue:String; override;
      procedure SetValue(Const value:String); override;
  end;

 { MemoName Property Editor Class}
 TMNPropertyEditor=class(tPropertyEditor)
     public
      procedure edit; override;
      function GetAttributes: TPropertyAttributes; override;
      function GetValue:String; override;
      procedure SetValue(Const value:String); override;
  end;

 { MasterIndexName Property Editor Class}
 TMIPropertyEditor=class(tPropertyEditor)
     public
      procedure edit; override;
      function GetAttributes: TPropertyAttributes; override;
      function GetValue:String; override;
      procedure SetValue(Const value:String); override;
  end;

 { Databasename Property Editor Class}
 TDNPropertyEditor=class(tPropertyEditor)
     public
      procedure edit; override;
      function GetAttributes: TPropertyAttributes; override;
      function GetValue:String; override;
      procedure SetValue(Const value:String); override;
  end;

  { Sortfields Property Editor Class }
  TSFPropertyEditor=class(tPropertyEditor)
     public
      procedure edit; override;
      function GetAttributes: TPropertyAttributes; override;
      function GetValue:String; override;
      procedure SetValue(Const value:String); override;
  end;

  { Groupfields Property Editor Class }
  TGFPropertyEditor=class(tPropertyEditor)
     public
      procedure edit; override;
      function GetAttributes: TPropertyAttributes; override;
      function GetValue:String; override;
      procedure SetValue(Const value:String); override;
  end;

  { UserParamsNames/Values Property Editor Class }
  TUSPropertyEditor=class(tPropertyEditor)
     public
      procedure edit; override;
      function GetAttributes: TPropertyAttributes; override;
      function GetValue:String; override;
      procedure SetValue(Const value:String); override;
  end;

   { RelationAlias/Tables/Indexs/Tags Property Editor Class }
  TTRPropertyEditor=class(tPropertyEditor)
     public
      procedure edit; override;
      function GetAttributes: TPropertyAttributes; override;
      function GetValue:String; override;
      procedure SetValue(Const value:String); override;
  end;

  { Filter Property Editor Class }
  TFIPropertyEditor=class(tPropertyEditor)
     public
      procedure edit; override;
      function GetAttributes: TPropertyAttributes; override;
      function GetValue:String; override;
      procedure SetValue(Const value:String); override;
  end;

  { Property Custom Types }
  TrrFilterUsage = (rrfuSaved,rrfuNone,rrfuComponent,rrfuInteractive);
  TrrDest = (rrdDisplay,rrdTextFile,rrdPrinter,rrdWorksheet,rrdXBase,
             rrdInteractive,rrdCSV,rrdMSWord,rrdRTF);
  TrrExportDest = (rredDisplay,rredFile,rredPrinter);
  TrrwsBorderStyle = (rrwsNone,rrwsFixedSingle,rrwsSizable,rrwsFixedDouble);

  {XBASE Specific Type definitions}
  TRRIndexExt = (rrieNone,rrieCDX,rrieIDX,rrieMDX,rrieNDX,rrieNSX,
                 rrieNTX,rrieWDX);
  TRRScopeUsage = (rrsuSaved,rrsuEntire,rrsuOverride,rrsuInteractive);
  TRRDataType = (rrdtNumeric,rrdtDate,rrdtCharacter);

  { Main Class Definition }
  TRRX65 = class(TComponent)
  private
    { Private declarations }

     { XBASE Specific Fields Follow }
     fAbout:Boolean;
     fAuthor:String;
     fIndexExtension:TRRIndexExt;
     fMasterIndexName:String;
     fMasterIndexTag:String;
     fMasterIndexType:TRRDataType;
     fRelationTables:Tstrings;
     fRelationIndexs:TStrings;
     fRelationTags:Tstrings;
     fRelationAlias:Tstrings;
     fScopeLow:String;
     fScopeHigh:String;
     fScopeUsage:TRRScopeUsage;
     fWriteAllow:Boolean;
     fXbaseEditor:Boolean;

     { Generic Fields Follow }
     fActive:Boolean;
     fAskPrinter:Boolean;
     fAskReport:Boolean;
     fAppName:String;
     fBeginPage: Longint;
     fCopies: Longint;
     fDatabasename: String;
     fDataDir    : String;
     fDisplayErrors: Boolean;
     fDisplayStatus: Boolean;
     fEndPage: LongInt;
     fErrorCode: String;
     fErrorMessage: String;
     fExportDest: TrrExportDest;
     fFields: TStrings;
     fFilter: String;
     fFilterLong:TStrings;
     fFilterUsage: TrrFilterUsage;
     fGroupFields: TStrings;
     fImageDir   : String;
     fLastErrorPage:Longint;
     fLibName: String;
     fMasterTableName: String;
     fMemoName: String;
     fOutputDest: TRRDest;
     fOutputFile: String;
     fPreventEscape: Boolean;
     fPrinterName: String;
     fPrinterPort: String;
     fRepName:String;
     fReportPick: Boolean;
     fSortFields: TStrings;
     fStatusEveryPage: Boolean;
     fSuppressTitle: Boolean;
     fTestPattern: Boolean;
     fUserParamsNames: TStrings;
     fUserParamsValues: TStrings;
     fVersion: String;
     fWait: Boolean;
     fWinBorderStyle: TrrwsBorderStyle;
     fWinControlBox: Boolean;
     fWinHeight: Integer;
     fWinLeft:Integer;
     fWinMaxButton: Boolean;
     fWinMinButton: Boolean;
     fWinParentHandle: Integer;
     fWinTitle: String;
     fWinTop: Integer;
     fWinWidth:Integer;

     { Event fields }
     fOnReportStart:TReportStartEvent;
     fOnReportEnd:TReportEndEvent;

     { XBASE Specific private functions follow }

     procedure setRelationTables(Value:Tstrings);
     procedure setRelationIndexs(Value:Tstrings);
     procedure setRelationTags(Value:Tstrings);
     procedure setRelationAlias(Value:Tstrings);
     procedure LoadRelations(hReport:Integer);
     procedure LoadScopes(hReport:Integer);
     procedure LoadIndex(hReport:Integer);
     procedure LoadFromReportH(hMyReport:Integer);

     { Generic functions follow }

     procedure setabout(Value:Boolean);
     procedure setfActive(Value:Boolean);
     procedure setauthor(Value:String);
     procedure LoadFields(hReport:Integer);
     procedure LoadGroupFields(hReport:Integer);
     procedure LoadSortFields(hReport:Integer);
     procedure LoadUserParams(hReport:Integer);
     procedure LoadTitle(hReport:Integer);
     procedure LoadPages(hReport:Integer);
     procedure LoadTable(hReport:Integer);
     procedure LoadFilter(hReport:Integer);
     procedure LoadDests(hReport:Integer);
     procedure LoadPrinter(hReport:Integer);

     procedure setfFields(Value:Tstrings);
     procedure setfFilter(Value:String);
     procedure setfFilterLong(Value:Tstrings);
     procedure setfGroupFields(Value:Tstrings);
     procedure setfSortFields(Value:Tstrings);
     procedure setfUserParamsNames(Value:Tstrings);
     procedure setfUserParamsValues(Value:Tstrings);
     procedure SetLibName(NewLibName:String);
     procedure SetRepName(NewRepName: String);
     procedure SetVersion(Value:String);
     procedure SaveToReportH(hReport:Integer);

     procedure SortForm(SortPage:Boolean);
     procedure SortFormSave(GroupForm:TSortDialog);
     procedure SortFormLoad(GroupForm:TSortDialog);


  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(Aowner:TComponent); override;
    destructor Destroy;Override;
    function execute:Boolean;virtual;
    function run:Boolean;virtual;
    procedure LoadFromReport;virtual;
    function SaveToFile(MyFile:String):Boolean;virtual;
    function LoadFromFile(MyFile:String):Boolean;virtual;
    procedure Clear;virtual;
    procedure Preview;virtual;

    procedure SelectReportName;virtual;
    procedure SelectReportLibrary;virtual;
    procedure SelectPrinter;virtual;
    procedure SelectDataDirectory;virtual;
    procedure SelectImageDirectory;virtual;
    procedure SelectMasterTable;virtual;
    procedure SelectMemoName;virtual;
    procedure SelectMasterIndexName;virtual;
    procedure SelectDatabaseName;virtual;
    procedure SelectSortFields;virtual;
    procedure SelectGroupFields;virtual;
    procedure SelectUserParams;virtual;
    procedure SelectRelations;virtual;
    procedure SelectFilter;virtual;
    procedure SelectDirs;virtual;

  published
    { Published declarations }

    { XBASE Specific Properties Follow }

    property IndexExtension:TRRIndexExt read fIndexExtension
                                        write fIndexExtension;
    property MasterIndexType:TRRDataType read  fMasterIndexType
                                         write fMasterIndexType;
    property MasterIndexName:String read fMasterIndexName
                                    write fMasterIndexName;
    property MasterIndexTag:String read fMasterIndexTag
                                   write fMasterIndexTag;
    property ScopeLow:String read fScopeLow write fScopeLow;
    property RelationTables:TStrings read fRelationTables write SetRelationTables;
    property RelationIndexs:TStrings read fRelationIndexs write SetRelationIndexs;
    property RelationTags:Tstrings read fRelationTags write SetRelationTags;
    property RelationAlias:Tstrings read fRelationAlias write setRelationAlias;
    property ScopeHigh:String read fScopeHigh write fScopeHigh;
    property ScopeUsage:TRRScopeUsage read fScopeUsage write fScopeUsage;
    property WriteAllow:Boolean read fWriteAllow write fWriteAllow;
    property XbaseEditor:Boolean read fXbaseEditor write fXbaseEditor;

    { Generic Properties follow }

    property About:Boolean read fAbout write SetAbout;
    property Active: Boolean read fActive write SetfActive;
    property AppName: String read fAppName write fAppName;
    property AskPrinter:Boolean read fAskPrinter write fAskPrinter;
    property AskReport:Boolean read fAskReport write fAskReport;
    property Author:String read fAuthor write SetAuthor;
    property BeginPage: Longint read fBeginPage write fBeginPage;
    property Copies: Longint read fCopies write fCopies;
    property Databasename:String read fDatabasename write fDatabasename;
    property DataDirectory:String read fDataDir write fDataDir;
    property DisplayErrors: Boolean read fDisplayErrors write fDisplayErrors;
    property DisplayStatus: Boolean read fDisplayStatus write fDisplayStatus;
    property EndPage: Longint read fEndPage write fEndPage;
    property ErrorCode: String read fErrorcode write fErrorcode;
    property ErrorMessage: String read fErrorMessage write fErrorMessage;
    property ExportDest: TrrExportDest read fExportDest write fExportDest;
    property Fields: TStrings read fFields write setfFields;
    property Filter: String read fFilter write setfFilter;
    property FilterLong: TStrings read fFilterLong write setfFilterLong;
    property FilterUsage: TrrFilterUsage read fFilterUsage write fFilterUsage;
    property GroupFields: TStrings read fGroupFields write setfGroupFields;
    property ImageDir: String read fImageDir write fImageDir;
    property LastErrorPage: Longint read fLastErrorPage write fLastErrorPage;
    property MasterTableName: String read fMasterTableName write fMasterTableName;
    property MemoName: String read fMemoName write fMemoName;
    property OutputDest: TrrDest read fOutputDest write fOutputDest;
    property OutputFile: String read fOutputFile write fOutputFile;
    property PreventEscape: Boolean read fPreventEscape write fPreventEscape;
    property PrinterName: String read fPrinterName write fPrinterName;
    property PrinterPort: String read fPrinterPort write fPrinterPort;
    property ReportLibrary:String read fLibName write setLibName;
    property ReportName: String read fRepName write setRepName;
    property SortFields: TStrings read fSortFields write setfSortFields;
    property StatusEveryPage: Boolean read fStatusEveryPage
                                      write fStatusEveryPage;
    property SuppressTitle: Boolean read fSuppressTitle write fSuppressTitle;
    property TestPattern: Boolean read fTestPattern write fTestPattern;
    property UserParamsNames: TStrings read fUserParamsNames
                                       write setfUserParamsNames;
    property UserParamsValues: TStrings read fUserParamsValues
                                       write setfUserParamsValues;
    property Version:String read fVersion write setversion;
    property Wait: Boolean read fWait write fWait;
    property WinBorderStyle:TrrwsBorderStyle read fWinBorderStyle
             write fWinBorderStyle;
    property WinControlBox: Boolean read fWinControlBox write fWinControlBox;
    property WinHeight: Integer read fWinHeight write fWinHeight;
    property WinLeft: Integer read fWinLeft write fWinLeft;
    property WinMaxButton: Boolean read fWinMaxButton write fWinMaxButton;
    property WinMinButton: Boolean read fWinMinButton write fWinMinButton;
    property WinParentHandle: Integer read fWinParentHandle write fWinParentHandle;
    property WinTitle: String read fWinTitle write fWinTitle;
    property WinTop:Integer read fWinTop write fWinTop;
    property WinWidth:Integer read fWinWidth write fWinWidth;

    { Events follow }
    property OnReportStart:TReportStartEvent read fOnReportStart write fOnReportStart;
    property OnReportEnd:TReportEndEvent read fOnReportEnd write fOnReportEnd;

end;

procedure Register;

implementation

uses
  RRXAbt,Rrxrelat,RRXFil,rrxdir,rrxed;

{ ************************************************************************** }
constructor TRRX65.create(AOwner:Tcomponent);
{ ************************************************************************** }
begin
  inherited create(AOwner);

  initruntimeinstance;

  fversion:='2.2';
  fAuthor:='cbrooksbank@msn.com';

  { XBASE Specific TSTRING fields }
  fFilterLong:=TStringList.Create;
  fRelationTables:=TStringList.Create;
  fRelationIndexs:=TStringList.Create;
  fRelationTags:=TStringList.Create;
  fRelationAlias:=TStringList.Create;
  fXbaseEditor:=true;

  { Generic TSTRING fields }
  fFields:=TStringList.Create;
  fGroupFields:=TStringList.Create;
  fSortFields:=TstringList.Create;
  fUserParamsNames:=TStringList.Create;
  fUserParamsValues:=TStringList.Create;

  { Some defaults I thought looked sensible }
  fWait:=true;
  fMasterIndexType:=rrdtCharacter;
  fIndexExtension:=rrieNTX;
  fAppName:=Application.exename;
  fAskPrinter:=false;
  fLastErrorPage:=0;
  WinTitle:='Delphi/R&R Object - (C) Systematics International 1995,1996';
  WinControlBox:=true;
  WinBorderStyle:=rrwsSizable;
  WinMinButton:=true;
  WinMaxButton:=true;
  WinLeft:=30;
  WinTop:=30;
  WinWidth:=Screen.Width div 2;
  WinHeight:=Screen.Height div 2 ;
  XBaseEditor:=True;
  WriteAllow:=true;
  WinParentHandle:=0;

end;

{ ************************************************************************** }
destructor TRRX65.Destroy;
{ ************************************************************************** }
begin

  { XBASE Specific TSTRING fields }
  fRelationTables.Free;
  fRelationIndexs.Free;
  fRelationTags.Free;
  fRelationAlias.Free;

  { Generic TSTRING Fields }
  fFields.free;
  fGroupFields.free;
  fSortFields.free;
  fUserParamsNames.free;
  fUserParamsValues.free;
  fFilterLong.Free;

  {endruntimeinstance};

  inherited Destroy;
end;


{ ************************************************************************** }
procedure Register;
{ ************************************************************************** }
{ Register the component and all its custom property editors }
begin
  RegisterComponents('Data Access', [TRRX65]);
  RegisterComponentEditor(TRRX65,TRRX65Editor);
  RegisterPropertyEditor(TypeInfo(String),TRRX65,'ReportLibrary',
                         TRLPropertyEditor);
  RegisterPropertyEditor(TypeInfo(String),TRRX65,'ReportName',
                         TRNPropertyEditor);
  RegisterPropertyEditor(TypeInfo(String),TRRX65,'PrinterName',
                         TPRPropertyEditor);
  RegisterPropertyEditor(TypeInfo(String),TRRX65,'PrinterPort',
                         TPRPropertyEditor);
  RegisterPropertyEditor(TypeInfo(String),TRRX65,'DataDirectory',
                         TDDPropertyEditor);
  RegisterPropertyEditor(TypeInfo(String),TRRX65,'ImageDir',
                         TIDPropertyEditor);
  RegisterPropertyEditor(TypeInfo(String),TRRX65,'MasterTableName',
                         TMTPropertyEditor);
  RegisterPropertyEditor(TypeInfo(String),TRRX65,'MemoName',
                         TMNPropertyEditor);
  RegisterPropertyEditor(TypeInfo(String),TRRX65,'MasterIndexName',
                         TMIPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings),TRRX65,'SortFields',
                         TSFPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings),TRRX65,'GroupFields',
                         TGFPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings),TRRX65,'UserParamsNames',
                         TUSPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings),TRRX65,'UserParamsValues',
                         TUSPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings),TRRX65,'RelationAlias',
                         TTRPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings),TRRX65,'RelationTables',
                         TTRPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings),TRRX65,'RelationIndexs',
                         TTRPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings),TRRX65,'RelationTags',
                         TTRPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings),TRRX65,'Filter',
                         TFIPropertyEditor);
end;

{ ************************************************************************** }
procedure TRRX65.SetAuthor(Value:String);
{ ************************************************************************** }
{ Dont allow author to change ! }
var
  designing:Boolean;
begin
  Designing:=(csDesigning in ComponentState);
  if ((csReading in ComponentState) or (csLoading in ComponentState)) then
    Designing:=false;

  fAuthor:='cbrooksbank@msn.com';
   if Designing then
    MessageDlg('Please send bugs/questions/suggestions to cbrooksbank@msn.com',
               mtinformation,[mbok],0);
end;


{ ************************************************************************** }
procedure TRRX65.LoadFromReport;
{ ************************************************************************** }
{ Using the fRepname and fLibname load object info from report defaults }
var
  hMyReport:Integer;
  MyApp_,MyLib_,MyRep_:PChar;
begin
  if ((flibname<>'') and (frepname<>'')) then
  begin
    MyApp_:=StrAlloc(length(fAppName)+1);
    StrPCopy(MyApp_,fAppName);
    MyLib_:=StrAlloc(length(fLibName)+1);
    StrPCopy(MyLib_,fLibName);
    MyRep_:=StrAlloc(length(fRepName)+1);
    StrPCopy(MyRep_,fRepName);
    try

      hMyReport:=ChooseReport(MyApp_,MyLib_,MyRep_,StrBufSize(MyRep_));
      if hMyReport<1 then
        raise ErrHandleAllocation.Create('Cant load info on R&&R report '+ReportName);

      try
        LoadFromReportH(hMyReport);
      finally
        EndReport(hMyReport);
      end;

    finally
      StrDispose(MyApp_);
      StrDispose(MyLib_);
      StrDIspose(MyRep_);
    end;
  end;
end;


{ ************************************************************************** }
procedure TRRX65.LoadFromReportH(hMyReport:Integer);
{ ************************************************************************** }

{ Using passed report handle load object info from report defaults }

begin

  { Clear XBASE Specific TSTRING Fields }
  fRelationTables.Clear;
  fRelationIndexs.Clear;
  fRelationTags.Clear;
  fRelationAlias.Clear;

  { Clear Generic TSTRING Fields }
  fFields.Clear;
  fGroupFields.Clear;
  fSortFields.Clear;
  fUserParamsNames.Clear;
  fUserParamsValues.Clear;
  fFilterLong.Clear;

  { Load XBASE Specific Properties }
  LoadRelations(hMyReport);
  LoadScopes(hMyReport);
  LoadIndex(hMyReport);

  { Load Generic Properties }
  LoadFields(hMyReport);
  LoadGroupFields(hMyReport);
  LoadSortFields(hMyReport);
  LoadUserParams(hMyReport);
  LoadTitle(hMyReport);
  LoadPages(hMyReport);
  LoadTable(hMyReport);
  LoadFilter(hMyReport);
  LoadDests(hMyReport);
  LoadPrinter(hMyReport);

  OutputFile:='';

end;

{ ************************************************************************** }
procedure TRRX65.setlibname(NewLibName:String);
{ ************************************************************************** }
{ Set method for ReportLibrary property }
var
  Designing:Boolean;
begin
  flibname:=NewLibName;
  Designing:=(csDesigning in ComponentState);
  if ((csReading in ComponentState) or (csLoading in ComponentState)) then
    Designing:=false;

  if (Designing and (fLibName<>'') and (fRepName<>'') ) then
  begin
    if MessageDlg('Load Properties From Report ?',mtconfirmation,
                  [mbNo,mbYes],0)=mrYes then loadfromreport;
  end;
end;

{ ************************************************************************** }
function TRRX65.Run:Boolean;
{ ************************************************************************** }
{ Run method just calls execute }
begin
  Result:=Execute;
end;


{ ************************************************************************** }
function TRRX65.Execute:Boolean;
{ ************************************************************************** }
{ Run the report }
var
  { Handle of report }
  hReport: Integer;

  { Flags returned after report was run }
  ECode:Integer;
  cmdshow: Integer;
  PageCount:LongInt;
  EMsg:Pchar;
  ErrorMess: String;

  AppName_,LibName_,RepName_:Pchar;
  Designing,ExecResult,Allow2Run:Boolean;
begin
  if ReportName='' then SelectReportName;

  {Run the report }
  Result:=false;

  { Convert Pascal type strings to C++ strings as expected by DLL }
  AppName_:=StrAlloc(length(fAppName)+1);
  LibName_:=StrAlloc(length(fLibName)+1);
  RepName_:=StrAlloc(length(fRepName)+1);
  StrPCopy(AppName_,fAppName);
  StrPCopy(LibName_,flibname);
  EMsg:=StrAlloc(256);
  if faskreport then StrPCopy(RepName_,'') else
    StrPCopy(RepName_,fRepName);

  try
    { If object specifys ask for report then blank out the report name }
    { to make sure user is prompted for report name }

    hReport:=ChooseReport(Appname_,LibName_,RepName_,StrBufSize(RepName_));

    if hReport>0 then begin
      try
        { Pass all the propertys to RSREPORT.DLL }
        SaveToReportH(hReport);

        { Run the report and then clean up }
        cmdshow:=SW_SHOWNORMAL;
        fErrorCode:='';
        fErrorMessage:='';
        ResetErrorInfo;

        { Run custom start event, report then custom end event }
        Allow2Run:=true;
        if Assigned(fOnReportStart) then fOnReportStart(hReport,Allow2Run);
        if (not Allow2Run) then Raise ErrCodeAbort.Create('');
        ExecResult:= ExecRunTime(hReport,fWait,cmdshow,@ECode,
                     @PageCount,EMsg,StrBufSize(EMsg));
        if Assigned(fOnReportEnd) then fOnReportEnd(hReport);

        if ExecResult then Result:=True else begin
          geterrorinfo(Emsg,StrBufSize(EMsg),@Ecode);
          fErrorMessage:=StrPas(EMsg);

          case Ecode of
            Ord('C'):fErrorCode:='C';
            Ord('D'):fErrorCode:='D';
            Ord('I'):fErrorCode:='I';
            Ord('J'):fErrorCode:='J';
            Ord('L'):fErrorCode:='L';
            Ord('S'):fErrorCode:='S';
            Ord('V'):fErrorCode:='V';
          else
            fErrorCode:=Chr(Ecode);
          end;

          case Ecode of
            Ord('C'):raise ErrCancel.Create(fErrorMessage);
            Ord('D'):raise ErrDiagnostic.Create(fErrorMessage);
            Ord('I'):raise ErrIteration.Create(fErrorMessage);
            Ord('J'):raise ErrJobControl.Create(fErrorMessage);
            Ord('L'):raise ErrLibrary.Create(fErrorMessage);
            Ord('S'):raise ErrSyntax.Create(fErrorMessage);
            Ord('V'):raise ErrValue.Create(fErrorMessage);
          else
            raise ErrUnknown.Create(fErrorMessage);
          end;

        end;
      finally
        EndReport(hReport);
      end;
    end;
    fLastErrorPage:=PageCount;
    if hReport<1 then
      raise ErrHandleAllocation.Create('Cant allocate handle for report '+fRepname);
  finally
    StrDispose(AppName_);
    StrDispose(LibName_);
    StrDispose(RepName_);
    StrDispose(EMsg);
  end;
end;

{ ************************************************************************** }
procedure TRRX65.LoadUserParams(hReport:Integer);
{ ************************************************************************** }
{ Load user paramaters from report with given handle }
var
  ParamName,ParamValue:Pchar;
begin
  ParamName:=StrAlloc(31);
  ParamValue:=StrAlloc(31);
  try
    if GetFirstUserParam(hReport,ParamName,StrBufSize(ParamName),
                      paramValue,StrBufSize(ParamValue)) then begin
      fUserParamsNames.Add(StrPas(ParamName));
      fUserParamsValues.Add(StrPas(ParamValue));
    end;
    while GetNextUserParam(hReport,ParamName,StrBufSize(ParamName),
                           ParamValue,StrBufSize(ParamValue)) do begin
      fUserParamsNames.Add(StrPas(ParamName));
      fUSerParamsValues.Add(StrPas(ParamValue));
    end;
  finally
    StrDispose(ParamName);
    StrDispose(ParamValue);
  end;
end;

{ ************************************************************************** }
procedure TRRX65.LoadFields(hReport:Integer);
{ ************************************************************************** }
{ Load all fields given report handle }
{ Note altering this property has no effect on report }
var
  FieldName:PChar;
begin
  FieldName:=StrAlloc(31);
  try
    if GetFirstFieldName(hReport,FieldName,StrBufSize(FieldName)) then
      fFields.Add(StrPas(FieldName));
    while GetNextFieldName(hReport,FieldName,StrBufSize(FieldName)) do
      fFields.Add(StrPas(FieldName));
  finally
    StrDispose(FieldName);
  end;
end;

{ ************************************************************************** }
procedure TRRX65.LoadGroupFields(hReport:Integer);
{ ************************************************************************** }
{ Load group fields given report handle }
var
  GroupField:Pchar;
begin
  GroupField:=StrAlloc(31);
  try
    if GetFirstGroupField(hReport,GroupField,StrBufSize(GroupField)) then
      fGroupFields.Add(StrPas(GroupField));
    while GetNextGroupField(hReport,GroupField,StrBufSize(GroupField)) do
      fGroupFields.Add(StrPas(GroupField));
  finally
    StrDispose(GroupField);
  end;
end;

{ ************************************************************************** }
procedure TRRX65.LoadSortFields(hReport:Integer);
{ ************************************************************************** }
{ Load sortfields given report handle }
var
  SortField:Pchar;
begin
  SortField:=StrAlloc(31);
  try
    if GetFirstSortField(hReport,SortField,StrBufSize(SortField)) then
      fSortFields.Add(StrPas(SortField));
    while GetNextSortField(hReport,SortField,StrBufSize(SortField)) do
      fSortFields.Add(StrPas(SortField));
  finally
    StrDispose(SortField);
  end;
end;

{ ************************************************************************** }
procedure TRRX65.SetRepName(NewRepName: String);
{ ************************************************************************** }
{ Set method for reportname property }
var
  Designing:Boolean;
begin
  fRepName:=NewRepName;
  Designing:=(csDesigning in ComponentState);
  if ((csReading in ComponentState) or (csLoading in ComponentState)) then
    Designing:=false;

  if (Designing and (fLibName<>'') and (fRepName<>'') ) then
  begin
    if MessageDlg('Load Properties From Report ?',mtconfirmation,
                  [mbYes,mbNo],0)=mrYes then loadfromreport;
  end;
end;

{ ************************************************************************** }
procedure TRRX65.SetVersion(Value: String);
{ ************************************************************************** }
{ Dont allow anybody to change the version number }
begin
  fVersion:='2.2';
end;

{ ************************************************************************** }
procedure TRRX65.SetfFields(Value:Tstrings);
{ ************************************************************************** }
{ Set method for fields property. This has no effect on report }
begin
  fFields.Assign(Value);
end;

{ ************************************************************************** }
procedure TRRX65.SetfFilter(Value:String);
{ ************************************************************************** }
{ Set method for filter property. Also synchs FilterLong }
begin
  fFilter:=Value;
  if fFilterLong.Count=0 then
    fFilterLong.Add(Value)
  else
    fFilterLong[0]:=Value;
end;

{ ************************************************************************** }
procedure TRRX65.SetfGroupFields(Value:Tstrings);
{ ************************************************************************** }
{ Set method for GroupFields property. This controls grouping of report. }
begin
  fGroupFields.Assign(Value);
end;

{ ************************************************************************** }
procedure TRRX65.SetfFilterLong(Value:Tstrings);
{ ************************************************************************** }
{ Set method for FilterLong property. This controls query of report.
  synchs with ffilter }
begin
  fFilterLong.Assign(Value);
  if fFilterLong.Count=0 then
    fFilter:=''
  else
    fFilter:=fFilterLong[0];
end;

{ ************************************************************************** }
procedure TRRX65.SetfSortFields(Value:Tstrings);
{ ************************************************************************** }
{ Set method for SortFields property. This controls sorting of report. }
begin
  fSortFields.Assign(Value);
end;

{ ************************************************************************** }
procedure TRRX65.SetfUserParamsNames(Value:Tstrings);
{ ************************************************************************** }
{ Set method for UserParamsNames property }
begin
  fUserParamsNames.Assign(Value);
end;

{ ************************************************************************** }
procedure TRRX65.SetfUserParamsValues(Value:Tstrings);
{ ************************************************************************** }
{ Set method for UserParamsValues property }
begin
  fUserParamsValues.Assign(Value);
end;

{ ************************************************************************** }
procedure TRRX65.SetAbout(Value:Boolean);
{ ************************************************************************** }
{ Dummy set method for About property. Pops up about box with toolbar. }
var
  AboutBox:TRRXAbout;
  Designing:Boolean;
begin
  Designing:=(csDesigning in ComponentState);
  if ((csReading in ComponentState) or (csLoading in ComponentState)) then
    Designing:=false;

  if Designing then begin
    AboutBox:=TRRXAbout.Create(Application);
    AboutBox.Report:=self;
    AboutBox.ShowModal;
    AboutBox.Free;
  end;
end;

{ ************************************************************************** }
procedure TRRX65.SetfActive(Value:Boolean);
{ ************************************************************************** }
{ Dummy set method for Active property. Runs report.}
begin
  if (csDesigning in ComponentState) and (Value=True) then execute;
end;

{ ************************************************************************** }
procedure TRRX65.LoadTitle(hReport:Integer);
{ ************************************************************************** }
{ Get method for WinTitle property. }
var
  title_:PChar;
begin
  title_:=StrAlloc(256);
  try
    if GetWinTitle(hReport,title_,StrBufSize(title_)) then
      fWinTitle:=StrPas(title_);
  finally
    StrDispose(title_);
  end;
end;

{ ************************************************************************** }
procedure TRRX65.LoadPages(hReport:Integer);
{ ************************************************************************** }
{ Load beginpage, EndPage and Copies properties from report handle }
begin
  GetBeginPage(hReport,@fBeginPage);
  GetEndPage(hReport,@fEndPage);
  GetCopies(hReport,@fCopies);
end;

{ ************************************************************************** }
procedure TRRX65.LoadTable(hReport:Integer);
{ ************************************************************************** }
{ Load mastertablename property from report handle }
var
  table_:Pchar;
begin
  table_:=StrAlloc(256);
  try
    if GetMasterTableName(hReport,table_,StrBufSize(table_)) then
      fMasterTableName:=StrPas(table_);
  finally
    StrDispose(table_);
  end;
end;


{ ************************************************************************** }
procedure TRRX65.LoadFilter(hReport:Integer);
{ ************************************************************************** }
{ Load filter from report handle }
var
  filter_:Pchar;
begin
  filter_:=StrAlloc(256);
  StrPCopy(filter_,'');
  try
    if getFilter(hReport,filter_,StrBufSize(filter_)) then begin
      fFilter:=StrPas(filter_);
      fFilterLong.Add(fFilter);
    end;
  finally
    StrDispose(Filter_);
  end;
end;

{ ************************************************************************** }
procedure TRRX65.LoadDests(hReport:Integer);
{ ************************************************************************** }
{ Load output and export destination propertys from report handle }
var
  Dest_:Pchar;
  DestStr:String;
begin

Dest_:=StrAlloc(256);
try

  { Load fExportDest}
  if getExportDest(hReport,Dest_) then
  begin
    DestStr:=StrPas(Dest_);
    if DestStr[1]='D' then
      fExportDest:=rredDisplay;
    if DestStr[1]='F' then
      fExportDest:=rredFile;
    if DestStr[1]='P' then
      fExportDest:=rredPrinter;
  end;

  { Load fOutPutDest }
  if getOutputDest(hReport,Dest_) then
  begin
    DestStr:=StrPas(Dest_);
    if DestStr[1]='D' then
      fOutPutDest:=rrdDisplay;
    if DestStr[1]='A' then
      fOutPutDest:=rrdTextFile;
    if DestStr[1]='T' then
      fOutPutDest:=rrdTextFile;
    if DestStr[1]='P' then
      fOutPutDest:=rrdPrinter;
    if DestStr[1]='W' then
      fOutPutDest:=rrdWorksheet;
    if DestStr[1]='X' then
      fOutPutDest:=rrdXBase;
    if DestStr[1]='?' then
      fOutPutDest:=rrdInteractive;
  end;

  { Load fOutputFile }
  if getOutputDest(hReport,Dest_) then
    fOutPutFile:=StrPas(Dest_);
finally
  StrDispose(Dest_);
end;

end;

{ ************************************************************************** }
procedure TRRX65.LoadPrinter(hReport:Integer);
{ ************************************************************************** }
{ Load printername and printerport properties from report handle }
var
  Printer_,Port_:Pchar;
begin
  Printer_:=StrAlloc(256);
  Port_:=StrAlloc(256);
  try
    if getPrinter(hReport,Printer_,StrBufSize(Printer_)) then
      fPrinterName:=StrPas(Printer_);
    if getPrinterPort(hReport,Port_,StrBufSize(Port_)) then
      fPrinterPort:=StrPas(Port_);
  finally
    StrDispose(Printer_);
    StrDispose(Port_);
  end;
end;

{ ************************************************************************** }
procedure TRRX65.SaveToReportH(hReport:Integer);
{ ************************************************************************** }

{ Given a report handle save object properties to the report }
{ Pass all the propertys to RSREPORT.DLL }

var
  { Temp vars to hold Pchar versions of String properties }
  PrinterName_:PChar;
  PrinterPort_:PChar;
  filter_,filterbit_:PChar;
  MasterTableName_,MemoName_,OutputFile_,WinTitle_:PChar;
  Tablename_,Databasename_:PChar;

  CharString:String;
  SiField,SiField2:PChar;
  datadir_,imagedir_:PChar;

  i,maxi:Integer;

  { XBASE specific VARS }
  lowscope_,highscope_:PChar;
  MasterIndex_,MasterTag_:PChar;
  RelationTable_,RelationIndex_,RelationTag_,RelationAlias_:PChar;


begin

  PrinterName_:= StrAlloc(256);
  PrinterPort_:= StrAlloc(31);

  filter_:= StrAlloc(1024);
  filterbit_:=StrAlloc(256);

  MasterTableName_:=StrAlloc(256);
  MemoName_:=StrAlloc(256);
  OutputFile_:=StrAlloc(256);
  WinTitle_:=StrAlloc(256);
  Tablename_:=StrAlloc(256);
  DatabaseName_:=StrAlloc(256);
  SiField:=StrAlloc(256);
  SiField2:=StrAlloc(256);
  datadir_:=StrAlloc(256);
  imagedir_:=StrAlloc(256);
  lowscope_:=StrAlloc(256);
  highscope_:=StrAlloc(256);
  MasterIndex_:=StrAlloc(256);
  MasterTag_:=StrAlloc(256);
  RelationTable_:=StrAlloc(256);
  RelationIndex_:=StrAlloc(256);
  RelationTag_:=StrAlloc(256);
  RelationAlias_:=StrAlloc(256);
  try

    if AskPrinter then fPrinterName:='?';

    { Convert Pascal type strings to C++ strings as expected by DLL }
    if fFilterLong.count=0 then
      StrPCopy(Filter_,fFilter)
    else begin
      StrPCopy(Filter_,fFilterLong[0]);
      for i:=1 to (fFilterLong.Count-1) do begin
        StrPCopy(FilterBit_,fFilterLong[i]);
        StrLCat(Filter_,filterbit_,1024);
      end;
    end;

    StrPCopy(MasterTableName_,fMasterTableName);
    StrPCopy(MemoName_,fMemoName);
    StrPCopy(OutputFile_,fOutputFile);
    StrPCopy(Printername_,fPrinterName);
    StrPCopy(PrinterPort_,fPrinterPort);
    StrPCopy(WinTitle_,fWinTitle);
    StrPCopy(Databasename_,fDatabasename);
    StrPCopy(ImageDir_,fimagedir);
    StrPCopy(DataDir_,fdatadir);

    { XBASE Specific StrPcopys }
    StrPcopy(lowscope_,fscopelow);
    StrPCopy(highscope_,fscopehigh);
    StrPCopy(Masterindex_,fMasterIndexName);
    StrPCopy(Mastertag_,fMasterIndexTag);

    SetBeginPage(hReport,fBeginPage);
    SetCopies(hReport,fCopies);
    SetDisplayErrors(hReport,fDisplayErrors);
    SetDisplayStatus(hReport,FDisplayStatus);
    SetEndPage(hReport,fEndPage);

    CharString:=Copy('DFP',Ord(fExportDest)+1,1);
    SetExportDest(hReport,CharString[1]);

    SetFilter(hReport,Filter_);

    CharString:=Copy('SEO?',Ord(fFilterUsage)+1,1);
    SetFilterUsage(hReport,CharString[1]);

    if fMasterTableName<>''then
      SetMasterTableName(hReport,MasterTableName_);

    if fMemoName<>'' then
      SetMemoName(hReport,MemoName_);

    CharString:=Copy('DAPWX?CMR',Ord(fOutputDest)+1,1);
    SetOutputDest(hReport,CharString[1]);

    SetOutPutFile(hReport,OutputFile_);
    SetPreventEscape(hReport,fPreventEscape);
    SetStatusEveryPage(hReport,fStatusEveryPage);
    SetTestPattern(hReport,fTestPattern);

    CharString:=Copy('0123',Ord(fWinBorderStyle)+1,1);
    SetWinBorderStyle(hReport,Ord(CharString[1])-Ord('0'));

    SetWinControlBox(hReport,fWinControlBox);
    SetWinHeight(hReport,fWinHeight);
    SetWinLeft(hReport,fWinLeft);
    SetWinMaxButton(hReport,fWinMaxButton);
    SetWinMinButton(hReport,fWinMinButton);
    SetWinParentHandle(hReport,fWinParentHandle);
    SetWinTitle(hReport,WinTitle_);
    SetWinTop(hReport,fWinTop);
    SetWinWidth(hReport,fWinWidth);

    { Set sort fields }
    for i:=0 to (fSortFields.Count-1) do begin
      if ((fSortFields[i]<>'') and
         (Pos('RECNO',UpperCase(fSortFields[i]))=0)) then begin
        StrPCopy(SiField,fSortFields[i]);
        SetSortField(hReport,SiField,i+1);
      end;
    end;

    { Set Group Fields }
    for i:=0 to (fGroupFields.Count-1) do begin
      if fGroupFields[i]<>'' then begin
        StrPCopy(SiField,fGroupFields[i]);
        SetGroupField(hReport,SiField,i+1);
      end;
    end;

    { Set user paramaters }
    for i:=0 to (fUserParamsNames.Count-1) do begin
      if fUserParamsNames[i]<>'' then begin
        StrPCopy(SiField,fUserParamsNames[i]);
        StrPCopy(SiField2,fUserParamsValues[i]);
        SetUserParam(hreport,SiField,SiField2);
      end;
    end;


    if ((fPrinterName<>'') or AskPrinter) then
    begin
      SetPrinter(hReport,PrinterName_);
      SetPrinterPort(hReport,PrinterPort_);
    end;

    if fimagedir<>'' then SetImageDir(hreport,imagedir_);
    if fdatadir<>'' then SetDataDir(hreport,datadir_);

    { Set XBASE specific things }

    SetXBaseEditor(hReport,fXbaseEditor);
    SetWriteAllow(hReport,fWriteAllow);
    SetIndexExtension(hReport,Ord(fIndexExtension));

    CharString:=Copy('SEO?',Ord(fScopeUsage)+1,1);
    SetScopeUsage(hReport,CharString[1]);

    if fScopeLow<>'' then SetLowScope(hreport,LowScope_);
    if fScopeHigh<>'' then SetHighScope(hreport,HighScope_);

    if fMasterIndexName<>'' then begin
      CharString:=Copy('NDC',Ord(fMasterIndexType)+1,1);
      SetMasterIndexInfo(hReport,MasterIndex_,CharString[1],MasterTag_);
    end;

    { Set XBASE Relationships }
    for i:=0 to (fRelationTables.Count-1) do begin
      StrPCopy(RelationTable_,fRelationTables[i]);
      StrPCopy(RelationIndex_,fRelationIndexs[i]);
      StrPCopy(RelationTag_,fRelationTags[i]);
      StrPCopy(RelationAlias_,fRelationAlias[i]);
      SetRelationInfo(hReport,RelationTable_,RelationIndex_,RelationTag_,
                      RelationAlias_,i+1);
    end;
  finally
    StrDispose(PrinterName_);
    StrDispose(PrinterPort_);
    StrDispose(filter_);
    StrDispose(filterbit_);
    StrDispose(MasterTableName_);
    StrDispose(MemoName_);
    StrDispose(OutputFile_);
    StrDispose(WinTitle_);
    StrDispose(Tablename_);
    StrDispose(DatabaseName_);
    StrDispose(SiField);
    StrDispose(SiField2);
    StrDispose(datadir_);
    StrDispose(imagedir_);
    StrDispose(lowscope_);
    StrDispose(highscope_);
    StrDispose(MasterIndex_);
    StrDispose(MasterTag_);
    StrDispose(RelationTable_);
    StrDispose(RelationIndex_);
    StrDispose(RelationTag_);
    StrDispose(RelationAlias_);
  end;

end;


{ ************************************************************************** }
function TRRX65.SaveToFile(MyFile:String):Boolean;
{ ************************************************************************** }
{Save properties to runtime extended ASCII control file}
var
  hReport,fh:Integer;
  MyFile_,MyApp_,MyLib_,MyRep_:PChar;
begin
  MyFile_:=StrAlloc(256);
  MyApp_:=StrAlloc(256);
  MyLib_:=StrAlloc(256);
  MyRep_:=StrAlloc(256);
  try

    Result:=false;
    StrPCopy(MyApp_,fAppName);
    StrPCopy(MyLib_,fLibName);
    StrPCopy(MyRep_,fRepName);

    hReport:=ChooseReport(MyApp_,MyLib_,MyRep_,StrBufSize(MyRep_));
    try
      if hReport>0 then
      begin
        StrPCopy(MyFile_,MyFile);
        savetoreportH(hReport);

        fh:=filecreate(MyFile);
        if fh>0 then begin
          fileclose(fh);
          if writeRunTimeRecord(hReport,MyFile_) then result:=true;
        end;

      end;
    finally
      EndReport(hReport);
    end;
  finally
    StrDispose(MyFile_);
    StrDispose(MyApp_);
    StrDispose(MyLib_);
    StrDispose(MyRep_);
  end;
end;

{ ************************************************************************** }
function TRRX65.LoadFromFile(MyFile:String):boolean;
{ ************************************************************************** }
{Load properties from runtime extended ASCII control file}
var
  hReport:Integer;
  MyFile_,MyApp_,MyLib_,MyRep_:PChar;
begin
  MyFile_:=StrAlloc(256);
  MyApp_:=StrAlloc(256);
  MyLib_:=StrAlloc(256);
  MyRep_:=StrAlloc(256);
  try
    Result:=false;
    StrPCopy(MyApp_,fAppName);
    StrPCopy(MyLib_,fLibName);
    StrPCopy(MyRep_,fRepName);

    StrPCopy(MyFile_,MyFile);
    hReport:=getRunTimeRecord(MyApp_,MyFile_);
    if hReport>0 then
    begin
      try
        { Load report information into this object }
        Result:=true;
        loadfromreporth(hReport);
      finally
        EndReport(hReport);
      end;
    end;
  finally
    StrDispose(MyFile_);
    StrDispose(MyApp_);
    StrDispose(MyLib_);
    StrDispose(MyRep_);
  end;
end;

{ ************************************************************************** }
procedure TRRX65.Clear;
{ ************************************************************************** }
{ Clear all properties }
begin
  fMasterIndexName:='';
  fMasterIndexTag:='';
  fRelationTables.Clear;
  fRelationIndexs.Clear;
  fRelationTags.Clear;
  fRelationAlias.Clear;
  fScopeUsage:=rrsuSaved;
  fAskPrinter:=false;
  fBeginPage:=0;
  fEndPage:=0;
  fCopies:=1;
  fDatabaseName:='';
  fDataDir:='';
  fErrorCode:='';
  fErrorMessage:='';
  fFields.Clear;
  fFilter:='';
  fFilterUsage:=rrfuSaved;
  fGroupFields.Clear;
  fImageDir:='';
  fLastErrorPage:=0;
  fLibName:='';
  fMasterTableName:='';
  fMemoName:='';
  fOutPutDest:=rrdDisplay;
  fOutPutFile:='';
  fPrinterName:='';
  fPrinterPort:='';
  fRepName:='';
  fReportPick:=false;
  fSortFields.Clear;
  fUserParamsNames.Clear;
  fUserParamsValues.Clear;
  fFilterLong.Clear;
  fWinTitle:='';
end;

{ ************************************************************************** }
procedure TRRX65.Preview;
{ ************************************************************************** }
{ Preview the report to screen }
var
  oldprinter,oldport,oldoutputFile:String;
  olddest:TrrDest;
  oldwait,olddisplaystatus,oldstatuseverypage:Boolean;
begin
  oldprinter:=printername;
  oldport:=printerport;
  olddest:=outputdest;
  oldoutputfile:=outputfile;
  oldwait:=wait;
  olddisplaystatus:=displaystatus;
  oldstatuseverypage:=statuseverypage;

  { Clear out printername in case not known on this system }
  printername:='';
  printerport:='';
  outputfile:='';
  wait:=true;
  displaystatus:=true;
  statuseverypage:=true;

  outputdest:=rrdDisplay;
  execute;

  printername:=oldprinter;
  printerport:=oldport;
  outputdest:=olddest;
  OutputFile:=oldoutputfile;
  wait:=oldwait;
  displaystatus:=olddisplaystatus;
  statuseverypage:=oldstatuseverypage;
end;

{ ************************************************************************** }
procedure TRRX65.SetRelationTables(Value:Tstrings);
{ ************************************************************************** }
begin
  fRelationTables.Assign(Value);
end;

{ ************************************************************************** }
procedure TRRX65.SetRelationIndexs(Value:Tstrings);
{ ************************************************************************** }
begin
  fRelationIndexs.Assign(Value);
end;

{ ************************************************************************** }
procedure TRRX65.SetRelationTags(Value:Tstrings);
{ ************************************************************************** }
begin
  fRelationTags.Assign(Value);
end;

{ ************************************************************************** }
procedure TRRX65.SetRelationAlias(Value:Tstrings);
{ ************************************************************************** }
begin
  fRelationAlias.Assign(Value);
end;

{ ************************************************************************** }
procedure TRRX65.LoadRelations(hReport:Integer);
{ ************************************************************************** }
{ Load relation properties from report handle }
var
  Table_,Index_,Tag_,Alias_:Pchar;
begin
  Table_:=StrAlloc(256);
  Index_:=StrAlloc(256);
  Tag_:=StrAlloc(256);
  Alias_:=StrAlloc(256);
  try

    if GetFirstRelationInfo(hReport,Table_,StrBufSize(Table_),Index_,StrBufSize(Index_),
                          Tag_,StrBufSize(Tag_),Alias_,StrBufSize(Alias_)) then begin

      fRelationTables.Add(StrPas(Table_));
      fRelationIndexs.Add(StrPas(Index_));
      fRelationTags.Add(StrPas(Tag_));
      fRelationAlias.Add(StrPas(Alias_));
    end;
    while GetNextRelationInfo(hReport,Table_,StrBufSize(Table_),Index_,StrBufSize(Index_),
                            Tag_,StrBufSize(Tag_),Alias_,StrBufSize(Alias_)) do begin
      fRelationTables.Add(StrPas(Table_));
      fRelationIndexs.Add(StrPas(Index_));
      fRelationTags.Add(StrPas(Tag_));
      fRelationAlias.Add(StrPas(Alias_));
    end;
  finally
    StrDispose(Table_);
    StrDispose(Index_);
    StrDispose(Tag_);
    StrDispose(Alias_);
  end;
end;


{ ************************************************************************** }
procedure TRRX65.LoadScopes(hReport:Integer);
{ ************************************************************************** }
{ Load scopes from report handle }
var
  Scope:Pchar;
begin
  Scope:=StrAlloc(256);
  try
    if GetLowScope(hReport,Scope,StrBufSize(Scope)) then
      fScopeLow:=StrPas(Scope);
    if GetHighScope(hReport,Scope,StrBufSize(Scope)) then
      fScopeHigh:=StrPas(Scope);
  finally
    StrDispose(Scope);
  end;
end;

{ ************************************************************************** }
procedure TRRX65.LoadIndex(hReport:Integer);
{ ************************************************************************** }
var
  Index_,Type_,Tag_:Pchar;
  TypeString:String;
begin
  Index_:=StrAlloc(256);
  Type_:=StrAlloc(256);
  Tag_:=StrAlloc(256);
  try

    if GetMasterIndexInfo(hReport,Index_,StrBufSize(Index_),Type_,Tag_,
                          StrBufSize(Tag_)) then begin
      fMasterIndexName:=StrPas(Index_);
      fMasterIndexTag:=StrPas(Tag_);

      TypeString:=StrPas(Type_);
      if TypeString[1]='N' then
        MasterIndexType:=rrdtNumeric;
      if TypeString[1]='D' then
        MasterIndexType:=rrdtDate;
      if TypeString[1]='C' then
        MasterIndexType:=rrdtCharacter;

    end;
  finally
    StrDispose(Index_);
    StrDispose(Type_);
    StrDispose(Tag_);
  end;
end;

{ ************************************************************************** }
procedure TRRX65.SelectReportName;
{ ************************************************************************** }
{ allow user to choose a report }
var
  hReport:Integer;
  AppName_,ReportLibrary_,ReportName_:PChar;
begin
 if ReportLibrary='' then SelectReportLibrary;

  AppName_       := StrAlloc(length(AppName)+1);
  ReportLibrary_ := StrAlloc(length(ReportLibrary)+1);
  ReportName_    := StrAlloc(256);

  try
    StrPCopy(AppName_,AppName);
    StrPCopy(ReportLibrary_,ReportLibrary);
    StrPCopy(ReportName_,'');
    hReport:=ChooseReport(AppName_,ReportLibrary_,ReportName_,StrBufSize(ReportName_));

    if hReport>0 then begin
      ReportName:=StrPas(ReportName_);
      endReport(hReport);
    end;

  finally
    StrDispose(AppName_);
    StrDispose(ReportLibrary_);
    StrDispose(ReportName_);
  end;

end;

{ ************************************************************************** }
procedure TRRX65.SelectReportLibrary;
{ ************************************************************************** }
{ Allow user to select report library }
var
  MyOpenDialog:TOpenDialog;
begin
  MyOpenDialog:=TOpenDialog.Create(Application);
  MyOpenDialog.Filter:='Report Librarys|*.RP5';
  if MyOpenDialog.execute then ReportLibrary:=MyOpenDialog.Filename;
  MyOpenDialog.Free;
end;

{ ************************************************************************** }
procedure TRRX65.SelectPrinter;
{ ************************************************************************** }
{ Allow user to select report printer }
var
  hReport:Integer;
  PrinterName_,PrinterPort_,AppName_:PChar;
begin
 PrinterName_ := StrAlloc(256);
 PrinterPort_ := StrAlloc(256);
 AppName_     := StrAlloc(256);

 try
   StrPCopy(PrinterName_,PrinterName);
   StrPCopy(PrinterPort_,PrinterName);
   StrPCopy(AppName_,AppName);

   hReport:=GetNewReportHandle(AppName_);

   if hReport>0 then begin
     try
         if ChoosePrinter(hReport,PrinterName_,StrBufSize(PrinterName_),
                              PrinterPort_,StrBufSize(PrinterPort_))
         then begin
           PrinterName:=StrPas(PrinterName_);
           PrinterPort:=StrPas(PrinterPort_);
         end;
     finally
       endreport(hReport);
     end;
   end;

 finally
   StrDispose(PrinterName_);
   StrDispose(PrinterPort_);
   StrDispose(AppName_);
 end;
end;

{ ************************************************************************** }
procedure TRRX65.SelectDataDirectory;
{ ************************************************************************** }
{ Allow user to select datadirectory }
begin
  SelectDirectory(fDataDir,[],0);
end;

{ ************************************************************************** }
procedure TRRX65.SelectImageDirectory;
{ ************************************************************************** }
{ Allow user to select imagedirectory }
begin
  SelectDirectory(fImageDir,[],0);
end;

{ ************************************************************************** }
procedure TRRX65.SelectMasterTable;
{ ************************************************************************** }
{ Allow user to select MasterTable }
var
  MyOpenDialog:TOpenDialog;
begin
  MyOpenDialog:=TOpenDialog.Create(Application);
  MyOpenDialog.Filter:='XBASE|*.DBF';
  if MyOpenDialog.execute then MasterTableName:=MyOpenDialog.Filename;
  MyOpenDialog.Free;
end;

{ ************************************************************************** }
procedure TRRX65.SelectMemoName;
{ ************************************************************************** }
{ Allow user to select MemoName }
var
  MyOpenDialog:TOpenDialog;
begin
  MyOpenDialog:=TOpenDialog.Create(Application);
  MyOpenDialog.Filter:='Text Files|*.TXT|All Files|*.*';
  if MyOpenDialog.execute then MemoName:=MyOpenDialog.Filename;
  MyOpenDialog.Free;
end;

{ ************************************************************************** }
procedure TRRX65.SelectMasterIndexName;
{ ************************************************************************** }
{ Allow user to select MasterIndexName }
var
  MyOpenDialog:TOpenDialog;
begin
  MyOpenDialog:=TOpenDialog.Create(Application);
  MyOpenDialog.Filter:='FoxPro|*.CDX|FoxBASE+|*.IDX|dBASE IV|*.MDX|'+
                       'dBASE III|*.NDX|Clipper|*.NTX|SuccessWare|*.NSX|'+
                       'WordTech|*.WDX|All Files|*.*';
  if MyOpenDialog.execute then MasterIndexName:=MyOpenDialog.Filename;
  MyOpenDialog.Free;
end;

{ ************************************************************************** }
procedure TRRX65.SelectDatabaseName;
{ ************************************************************************** }
{ Allow user to select DatabaseName }
begin
  SelectDirectory(fDatabasename,[],0);
end;

{ ************************************************************************** }
procedure TRRX65.SelectSortFields;
{ ************************************************************************** }
{ Allow user to select SortFields }
begin
  SortForm(true);
end;


{ ************************************************************************** }
procedure TRRX65.SelectGroupFields;
{ ************************************************************************** }
{ Allow user to select GroupFields }
begin
  SortForm(false);
end;

{ ************************************************************************** }
procedure TRRX65.SortForm(SortPage:Boolean);
{ ************************************************************************** }
{ Present sortfields/groupfields form depending on SortPage }
var
  GroupForm:TSortDialog;
begin
  GroupForm:=TSortDialog.Create(Application);
  try
    GroupForm.SortGrpNoteBook.PageIndex:=(Ord(not SortPage));
    SortFormLoad(GroupForm);
    if GroupForm.ShowModal=mrOK then
      SortFormSave(GroupForm);
  finally
    GroupForm.Free;
  end;
end;

{ ************************************************************************** }
procedure TRRX65.SortFormLoad(GroupForm:TSortDialog);
{ ************************************************************************** }
{ Load the Sortfields/Groupfields properties into the custom property dialog }
var
  i,myptr:Integer;
  look4:String;
  MyCombo:TComboBox;
  MyCheck:TCheckBox;
begin
   with GroupForm do begin
    { Load field list into dialogs combo's }
    for i:=0 to (fields.count-1) do begin
      SortCombo1.items.add(fields[i]);
      SortCombo2.items.add(fields[i]);
      SortCombo3.items.add(fields[i]);
      SortCombo4.items.add(fields[i]);
      SortCombo5.items.add(fields[i]);
      SortCombo6.items.add(fields[i]);
      SortCombo7.items.add(fields[i]);
      SortCombo8.items.add(fields[i]);
      GroupCombo1.items.add(fields[i]);
      GroupCombo2.items.add(fields[i]);
      GroupCombo3.items.add(fields[i]);
      GroupCombo4.items.add(fields[i]);
      GroupCombo5.items.add(fields[i]);
      GroupCombo6.items.add(fields[i]);
      GroupCombo7.items.add(fields[i]);
      GroupCombo8.items.add(fields[i]);
    end;

    { start at itemindex 0}
    SortCombo1.itemindex:=0;
    SortCombo2.itemindex:=0;
    SortCombo3.itemindex:=0;
    SortCombo4.itemindex:=0;
    SortCombo5.itemindex:=0;
    SortCombo6.itemindex:=0;
    SortCombo7.itemindex:=0;
    SortCombo8.itemindex:=0;
    GroupCombo1.itemindex:=0;
    GroupCombo2.itemindex:=0;
    GroupCombo3.itemindex:=0;
    GroupCombo4.itemindex:=0;
    GroupCombo5.itemindex:=0;
    GroupCombo6.itemindex:=0;
    GroupCombo7.itemindex:=0;
    GroupCombo8.itemindex:=0;

    { Set correct Sortfield itemindex for each combo }
    for i:=0 to (SortFields.count-1) do begin
      if i<8 then begin
        case i of
          0 : begin MyCombo:=SortCombo1;MyCheck:=SortCheck1;end;
          1 : begin MyCombo:=SortCombo2;MyCheck:=SortCheck2;end;
          2 : begin MyCombo:=SortCombo3;MyCheck:=SortCheck3;end;
          3 : begin MyCombo:=SortCombo4;MyCheck:=SortCheck4;end;
          4 : begin MyCombo:=SortCombo5;MyCheck:=SortCheck5;end;
          5 : begin MyCombo:=SortCombo6;MyCheck:=SortCheck6;end;
          6 : begin MyCombo:=SortCombo7;MyCheck:=SortCheck7;end;
          7 : begin MyCombo:=SortCombo8;MyCheck:=SortCheck8;end;
        end;
        look4:=SortFields[i];
        myptr:=MyCombo.items.indexof(Copy(look4,2,255));
        if myptr>0 then MyCombo.itemindex:=myptr;
        MyCheck.Checked:=(Copy(look4,1,1)='+');
      end;
    end;

    { Set correct Groupfield itemindex for each combo }
    for i:=0 to (GroupFields.count-1) do begin
      if i<8 then begin
        case i of
          0 : MyCombo:=GroupCombo1;
          1 : MyCombo:=GroupCombo2;
          2 : MyCombo:=GroupCombo3;
          3 : MyCombo:=GroupCombo4;
          4 : MyCombo:=GroupCombo5;
          5 : MyCombo:=GroupCombo6;
          6 : MyCombo:=GroupCombo7;
          7 : MyCombo:=GroupCombo8;
        end;
        look4:=GroupFields[i];
        myptr:=MyCombo.items.indexof(look4);
        if myptr>0 then MyCombo.itemindex:=myptr;
      end;
    end;

  end;
end;

{ ************************************************************************** }
procedure TRRX65.SortFormSave(GroupForm:TSortDialog);
{ ************************************************************************** }
{ Save settings made on sortfields/groupfields dialog back to fields property}
var
  Ascend:Array[0..1] of Char;
begin
  Ascend[0]:='-';
  Ascend[1]:='+';
  SortFields.Clear;
  GroupFields.Clear;

  with GroupForm do begin
    if SortCombo1.ItemIndex>0 then
      SortFields.Add(Ascend[ord(SortCheck1.Checked)]+SortCombo1.Text);
    if SortCombo2.ItemIndex>0 then
      SortFields.Add(Ascend[ord(SortCheck2.Checked)]+SortCombo2.Text);
    if SortCombo3.ItemIndex>0 then
      SortFields.Add(Ascend[ord(SortCheck3.Checked)]+SortCombo3.Text);
    if SortCombo4.ItemIndex>0 then
      SortFields.Add(Ascend[ord(SortCheck4.Checked)]+SortCombo4.Text);
    if SortCombo5.ItemIndex>0 then
      SortFields.Add(Ascend[ord(SortCheck5.Checked)]+SortCombo5.Text);
    if SortCombo6.ItemIndex>0 then
      SortFields.Add(Ascend[ord(SortCheck6.Checked)]+SortCombo6.Text);
    if SortCombo7.ItemIndex>0 then
      SortFields.Add(Ascend[ord(SortCheck7.Checked)]+SortCombo7.Text);
    if SortCombo8.ItemIndex>0 then
      SortFields.Add(Ascend[ord(SortCheck8.Checked)]+SortCombo8.Text);

    if GroupCombo1.ItemIndex>0 then GroupFields.Add(GroupCombo1.Text);
    if GroupCombo2.ItemIndex>0 then GroupFields.Add(GroupCombo2.Text);
    if GroupCombo3.ItemIndex>0 then GroupFields.Add(GroupCombo3.Text);
    if GroupCombo4.ItemIndex>0 then GroupFields.Add(GroupCombo4.Text);
    if GroupCombo5.ItemIndex>0 then GroupFields.Add(GroupCombo5.Text);
    if GroupCombo6.ItemIndex>0 then GroupFields.Add(GroupCombo6.Text);
    if GroupCombo7.ItemIndex>0 then GroupFields.Add(GroupCombo7.Text);
    if GroupCombo8.ItemIndex>0 then GroupFields.Add(GroupCombo8.Text);
  end;

end;

{ ************************************************************************** }
procedure TRRX65.SelectUserParams;
{ ************************************************************************** }
{ Allow user to edit user paramaters }
var
  UserForm:TUserForm;
  i:Integer;
begin
  UserForm:=TUserForm.Create(Application);
  try

    { Load }
    for i:=0 to (UserParamsNames.Count-1) do begin
      UserForm.NamesList.lines.add(UserParamsNames[i]);
    end;
    for i:=0 to (UserParamsValues.Count-1) do begin
      UserForm.ValuesList.lines.add(UserParamsValues[i]);
    end;

    { Display }
    if UserForm.ShowModal=mrOK then begin
      { Save }
      UserParamsNames.Clear;
      for i:=0 to (UserForm.NamesList.lines.Count-1) do begin
        UserParamsNames.Add(UserForm.NamesList.lines[i]);
      end;
      UserParamsValues.Clear;
      for i:=0 to (UserForm.ValuesList.lines.Count-1) do begin
        UserParamsValues.Add(UserForm.ValuesList.lines[i]);
      end;
    end;

  finally
    UserForm.Free;
  end;
end;

{ ************************************************************************** }
procedure TRRX65.SelectRelations;
{ ************************************************************************** }
{ Allow user to edit relationships }
var
  RelationForm:TRelationForm;
  i,ListCount:Integer;
begin
  RelationForm:=TRelationForm.Create(Application);
  try
    with RelationForm do begin
      { load }
      MasterTableEdit.Text:=MasterTableName;
      MasterIndexEdit.Text:=MasterIndexName;
      IndexTypeCombo.ItemIndex:=Ord(MasterIndexType);

      ListCount:=RelationAlias.Count;
      if RelationTables.Count>ListCount then
        ListCount:=RelationTables.Count;
      if RelationIndexs.Count>ListCount then
        ListCount:=RelationIndexs.Count;
      if RelationTags.Count<ListCount then
        ListCount:=RelationTags.Count;

      for i:=0 to (RelationAlias.Count-1) do begin
        AliasCombo.Items.Add(RelationAlias[i]);
      end;
      for i:=1 to (ListCount-RelationAlias.Count) do begin
        AliasCombo.Items.Add('');
      end;

      for i:=0 to (RelationTables.Count-1) do begin
        TableList.Items.Add(RelationTables[i]);
      end;
      for i:=1 to (ListCount-RelationTables.Count) do begin
        TableList.Items.Add('');
      end;

      for i:=0 to (RelationIndexs.Count-1) do begin
        IndexList.Items.Add(RelationIndexs[i]);
      end;
      for i:=1 to (ListCount-RelationIndexs.Count) do begin
        IndexList.Items.Add('');
      end;

      for i:=0 to (RelationTags.Count-1) do begin
        TagList.Items.Add(RelationTags[i]);
      end;
      for i:=1 to (ListCount-RelationTags.Count) do begin
        TagList.Items.Add('');
      end;

      if ListCount>0 then begin
        AliasCombo.ItemIndex:=0;
        TableEdit.text:=TableList.Items[0];
        IndexEdit.text:=IndexList.Items[0];
        TagEdit.text:=TagList.Items[0];
      end;

      if ListCount=0 then begin
        AliasLabel.enabled:=false;
        AliasCombo.enabled:=false;
        TableLabel.enabled:=false;
        TableEdit.enabled:=false;
        TableBtn.enabled:=false;
        IndexLabel.enabled:=false;
        IndexEdit.enabled:=false;
        IndexBtn.enabled:=false;
        TagLabel.Enabled:=false;
        TagEdit.Enabled:=false;
      end;

      { display }
      if ShowModal=mrOK then begin

        { save }
        MasterTableName:=MasterTableEdit.Text;
        MasterIndexName:=MasterIndexEdit.Text;
        if IndexTypeCombo.ItemIndex=0 then MasterIndexType:=rrdtNumeric;
        if IndexTypeCombo.ItemIndex=1 then MasterIndexType:=rrdtDate;
        if IndexTypeCombo.ItemIndex=2 then MasterIndexType:=rrdtCharacter;

        RelationAlias.Clear;
        RelationTables.Clear;
        RelationIndexs.Clear;
        RelationTags.Clear;
        for i:=0 to (AliasCombo.Items.Count-1) do begin
          RelationAlias.Add(AliasCombo.Items[i]);
          RelationTables.Add(TableList.Items[i]);
          RelationIndexs.Add(IndexList.Items[i]);
          RelationTags.Add(TagList.Items[i]);
        end;

      end;

    end;
  finally
    RelationForm.Free;
  end;
end;

{ ************************************************************************** }
procedure TRRX65.SelectFilter;
{ ************************************************************************** }
{ Allow user to edit filter and scope }
var
  FilterForm:TRRXFilt;
begin
  FilterForm:=TRRXFilt.Create(Application);
  try

    { Load info into dialog }
    FilterForm.FilterMemo.lines.assign(FilterLong);
    if FilterUsage=rrfuSaved       then FilterForm.FilterUsageCombo.Itemindex:=0;
    if FilterUsage=rrfuComponent   then FilterForm.FilterUsageCombo.Itemindex:=1;
    if FilterUsage=rrfuInteractive then FilterForm.FilterUsageCombo.Itemindex:=2;

    if ScopeUsage=rrsuSaved       then FilterForm.ScopeUsageCombo.ItemIndex:=0;
    if ScopeUsage=rrsuEntire      then FilterForm.ScopeUsageCombo.ItemIndex:=1;
    if ScopeUsage=rrsuOverride    then FilterForm.ScopeUsageCombo.ItemIndex:=2;
    if ScopeUsage=rrsuInteractive then FilterForm.ScopeUsageCombo.ItemIndex:=3;

    FilterForm.ScopeLow.text:=ScopeLow;
    FilterForm.ScopeHigh.text:=ScopeHigh;

    { Display }
    if FilterForm.ShowModal=mrok then begin
      { Save }
      ScopeLow:=FilterForm.ScopeLow.text;
      ScopeHigh:=FilterForm.ScopeHigh.text;
      if FilterForm.ScopeUsageCombo.ItemIndex=0 then fScopeUsage:=rrsuSaved;
      if FilterForm.ScopeUsageCombo.ItemIndex=1 then fScopeUsage:=rrsuEntire;
      if FilterForm.ScopeUsageCombo.ItemIndex=2 then fScopeUsage:=rrsuOverride;
      if FilterForm.ScopeUsageCombo.ItemIndex=3 then fScopeUsage:=rrsuInteractive;

      ffilterlong.assign(FilterForm.FilterMemo.lines);
      if ffilterlong.count>0 then
        ffilter:=ffilterlong[0]
      else
        ffilter:='';
      if FilterForm.FilterUsageCombo.ItemIndex=0 then fFilterUsage:=rrfuSaved;
      if FilterForm.FilterUsageCombo.ItemIndex=1 then fFilterUsage:=rrfuComponent;
      if FilterForm.FilterUsageCombo.ItemIndex=2 then fFilterUsage:=rrfuInteractive;

    end;

  finally
    FilterForm.Free;
  end;
end;

{ ************************************************************************** }
procedure TRRX65.SelectDirs;
{ ************************************************************************** }
{ Allow user to edit default directories }
var
  DirForm:TRRXDirs;
begin
  DirForm:=TRRXDirs.Create(Application);
  try
    { load }
    DirForm.DataDir.text:=fDataDir;
    DirForm.ImageDir.text:=fImageDir;
    { display }
    if DirForm.ShowModal=mrok then begin
      { save }
      fDataDir:=DirForm.DataDir.text;
      fImageDir:=DirForm.ImageDir.text;
    end;
  finally
    DirForm.Free;
  end;
end;

{ ****************** Code For Property Editors Follows ******************** }

{ ReportLibrary Property Editor Code }
procedure TRLPRopertyEditor.edit;
var
  MyOpenDialog:TOpenDialog;
begin
  MyOpenDialog:=TOpenDialog.Create(Application);
  MyOpenDialog.Filter:='Report Librarys|*.RP5';
  if MyOpenDialog.execute then SetStrValue(MyOpenDialog.Filename);
  MyOpenDialog.Free;
end;
function TRLPropertyEditor.getattributes:TPropertyAttributes;
begin
  result:=[paDialog];
end;
function TRLPropertyEditor.GetValue:String;
begin
  Result:=GetStrValue;
end;
procedure TRLPropertyEditor.SetValue(Const Value:String);
begin
  SetStrValue(Value);
end;

{ Reportname Property Editor Code }
procedure TRNPRopertyEditor.edit;
begin
 TRRX65(GetComponent(0)).SelectReportName;
end;
function TRNPropertyEditor.getattributes:TPropertyAttributes;
begin
  result:=[paDialog];
end;
function TRNPropertyEditor.GetValue:String;
begin
  Result:=GetStrValue;
end;
procedure TRNPropertyEditor.SetValue(Const Value:String);
begin
  SetStrValue(Value);
end;

{ Printername/Printerport Property Editor Code }
procedure TPRPropertyEditor.edit;
begin
  TRRX65(GetComponent(0)).SelectPrinter;
end;
function TPRPropertyEditor.getattributes:TPropertyAttributes;
begin
  result:=[paDialog];
end;
function TPRPropertyEditor.GetValue:String;
begin
  Result:=GetStrValue;
end;
procedure TPRPropertyEditor.SetValue(Const Value:String);
begin
  SetStrValue(Value);
end;

{ Datadirectory Property Editor Code }
procedure TDDPropertyEditor.edit;
begin
  TRRX65(GetComponent(0)).SelectDataDirectory;
end;
function TDDPropertyEditor.getattributes:TPropertyAttributes;
begin
  result:=[paDialog];
end;
function TDDPropertyEditor.GetValue:String;
begin
  Result:=GetStrValue;
end;
procedure TDDPropertyEditor.SetValue(Const Value:String);
begin
  SetStrValue(Value);
end;


{ ImageDirectory property Editor Code }
procedure TIDPropertyEditor.edit;
begin
  TRRX65(GetComponent(0)).SelectImageDirectory;
end;
function TIDPropertyEditor.getattributes:TPropertyAttributes;
begin
  result:=[paDialog];
end;
function TIDPropertyEditor.GetValue:String;
begin
  Result:=GetStrValue;
end;
procedure TIDPropertyEditor.SetValue(Const Value:String);
begin
  SetStrValue(Value);
end;

{ MasterTable Property Editor Code }
procedure TMTPropertyEditor.edit;
begin
  TRRX65(GetComponent(0)).SelectMasterTable;
end;
function TMTPropertyEditor.getattributes:TPropertyAttributes;
begin
  result:=[paDialog];
end;
function TMTPropertyEditor.GetValue:String;
begin
  Result:=GetStrValue;
end;
procedure TMTPropertyEditor.SetValue(Const Value:String);
begin
  SetStrValue(Value);
end;

{ Memoname Property Editor Code }
procedure TMNPropertyEditor.edit;
begin
  TRRX65(GetComponent(0)).SelectMemoName;
end;
function TMNPropertyEditor.getattributes:TPropertyAttributes;
begin
  result:=[paDialog];
end;
function TMNPropertyEditor.GetValue:String;
begin
  Result:=GetStrValue;
end;
procedure TMNPropertyEditor.SetValue(Const Value:String);
begin
  SetStrValue(Value);
end;


{ Masterindexname Property Editor Code }
procedure TMIPropertyEditor.edit;
begin
  TRRX65(GetComponent(0)).SelectMasterIndexName;
end;
function TMIPropertyEditor.getattributes:TPropertyAttributes;
begin
  result:=[paDialog];
end;
function TMIPropertyEditor.GetValue:String;
begin
  Result:=GetStrValue;
end;
procedure TMIPropertyEditor.SetValue(Const Value:String);
begin
  SetStrValue(Value);
end;

{ Databasename property editor code }
procedure TDNPropertyEditor.edit;
begin
  TRRX65(GetComponent(0)).SelectDatabaseName;
end;
function TDNPropertyEditor.getattributes:TPropertyAttributes;
begin
  result:=[paDialog];
end;
function TDNPropertyEditor.GetValue:String;
begin
  Result:=GetStrValue;
end;
procedure TDNPropertyEditor.SetValue(Const Value:String);
begin
  SetStrValue(Value);
end;

{ Sortfields property editor code }
procedure TSFPropertyEditor.edit;
begin
  TRRX65(GetComponent(0)).SelectSortFields;
end;
function TSFPropertyEditor.getattributes:TPropertyAttributes;
begin
  result:=[paDialog];
end;
function TSFPropertyEditor.GetValue:String;
begin
  Result:='(TStrings)';
end;
procedure TSFPropertyEditor.SetValue(Const Value:String);
begin
  SetStrValue('(TStrings)');
end;

{ Groupfields property editor code }
procedure TGFPropertyEditor.edit;
begin
  TRRX65(GetComponent(0)).SelectGroupFields;
end;
function TGFPropertyEditor.getattributes:TPropertyAttributes;
begin
  result:=[paDialog];
end;
function TGFPropertyEditor.GetValue:String;
begin
  Result:='(TStrings)';
end;
procedure TGFPropertyEditor.SetValue(Const Value:String);
begin
  SetStrValue('(TStrings)');
end;

{ UserParamsNames/Values property editor code }
procedure TUSPropertyEditor.edit;
begin
  TRRX65(GetComponent(0)).SelectUserParams;
end;
function TUSPropertyEditor.getattributes:TPropertyAttributes;
begin
  result:=[paDialog];
end;
function TUSPropertyEditor.GetValue:String;
begin
  Result:='(TStrings)';
end;
procedure TUSPropertyEditor.SetValue(Const Value:String);
begin
  SetStrValue('(TStrings)');
end;

{ Relations property editor code }
procedure TTRPropertyEditor.edit;
begin
  TRRX65(GetComponent(0)).SelectRelations;
end;
function TTRPropertyEditor.getattributes:TPropertyAttributes;
begin
  result:=[paDialog];
end;
function TTRPropertyEditor.GetValue:String;
begin
  Result:='(TStrings)';
end;
procedure TTRPropertyEditor.SetValue(Const Value:String);
begin
  SetStrValue('(TStrings)');
end;


{ Filter property editor code }
procedure TFIPropertyEditor.edit;
begin
  TRRX65(GetComponent(0)).SelectFilter;
end;
function TFIPropertyEditor.getattributes:TPropertyAttributes;
begin
  result:=[paDialog];
end;
function TFIPropertyEditor.GetValue:String;
begin
  Result:=GetStrValue;
end;
procedure TFIPropertyEditor.SetValue(Const Value:String);
begin
  SetStrValue(Value);
end;

end.
